<?php

namespace Drupal\image_hover\Plugin\Field\FieldFormatter;

use Drupal\Core\Render\Element;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

/**
 * Formatter for image hover effects.
 *
 * @FieldFormatter(
 *   id = "image_hover",
 *   label = @Translation("Image with Hover"),
 *   field_types = {
 *     "image"
 *   },
 * )
 */
class HoverImageFormatter extends ImageFormatter implements ContainerFactoryPluginInterface {

  /**
   * Gives the transition speed options.
   */
  public static function transitionSpeeds() {
    return [
      '0s' => 'Instant',
      '0.1s' => '100 milliseconds',
      '0.2s' => '200 milliseconds',
      '0.3s' => '300 milliseconds',
      '0.4s' => '400 milliseconds',
      '0.5s' => '500 milliseconds',
    ];
  }

  /**
   * Gives the transition type options.
   */
  public static function transitionTypes() {
    return [
      'ease' => 'Ease In & Ease Out',
      'linear' => 'Linear',
      'ease-out' => 'Ease Out',
      'ease-in' => 'Ease In',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'primary_image' => 'first',
      'transition_speed' => '0.2s',
      'transition_type' => 'ease',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $defaults = $this->defaultSettings();
    return [
      'disclaimer' => [
        '#markup' => '<p>For this format to work, the field must be configured to accept at least two images, and the content must have at least two images set.</p>',
      ],
    ] + parent::settingsForm($form, $form_state) + [
      'primary_image' => [
        '#type' => 'select',
        '#title' => $this->t('Primary Image (shown before hovering)'),
        '#options' => [
          'first' => 'First',
          'second' => 'Second',
        ],
        '#default_value' => $this->getSetting('primary_image') ?: $defaults['primary_image'],
      ],
      'transition_speed' => [
        '#type' => 'select',
        '#title' => $this->t('Hover Transition Speed'),
        '#options' => $this::transitionSpeeds(),
        '#default_value' => $this->getSetting('transition_speed') ?: $defaults['transition_speed'],
      ],
      'transition_type' => [
        '#type' => 'select',
        '#title' => $this->t('Hover Transition Type'),
        '#options' => $this::transitionTypes(),
        '#default_value' => $this->getSetting('transition_type') ?: $defaults['transition_type'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $image_styles = image_style_options(FALSE);
    // Unset possible 'No defined styles' option.
    unset($image_styles['']);
    // Styles could be lost because of enabled/disabled modules that defines
    // their styles in code.
    $image_style_setting = $this->getSetting('image_style');
    if (isset($image_styles[$image_style_setting])) {
      $summary[] = t('Image style: @style', ['@style' => $image_styles[$image_style_setting]]);
    }
    else {
      $summary[] = t('Original image');
    }

    $link_types = [
      'content' => t('Linked to content'),
      'file' => t('Linked to file'),
    ];
    // Display this setting only if image is linked.
    $image_link_setting = $this->getSetting('image_link');
    if (isset($link_types[$image_link_setting])) {
      $summary[] = $link_types[$image_link_setting];
    }

    $defaults = $this->defaultSettings();
    $transition_speed = $this::transitionSpeeds();
    $transition_type = $this::transitionTypes();
    $summary[] = 'Transition: ' . $transition_speed[$this->getSetting('transition_speed') ?: $defaults['transition_speed']] . ' / ' . $transition_type[$this->getSetting('transition_type') ?: $defaults['transition_type']];

    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * Pieced together from FormatterBase and EntityReferenceFormatterBase.
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    // Default the language to the current content language.
    if (empty($langcode)) {
      $langcode = \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }

    $elements = $this->viewElements($items, $langcode);

    // If there are actual renderable children, use #theme => field, otherwise,
    // let access cacheability metadata pass through for correct bubbling.
    if (Element::children($elements)) {
      $entity = $items->getEntity();
      $entity_type = $entity->getEntityTypeId();
      $field_name = $this->fieldDefinition->getName();
      $defaults = $this->defaultSettings();

      if ($this->getSetting('primary_image') === 'second') {
        list($secondary, $primary) = [reset($elements), next($elements)];
      }
      else {
        list($primary, $secondary) = [reset($elements), next($elements)];
      }

      $elements = [
        '#attached' => ['library' => ['image_hover/hoverable_image_field']],
        '#theme' => 'field_hover_image',
        '#label' => $this->fieldDefinition->getLabel(),
        '#label_display' => $this->label,
        '#label_hidden' => $this->label === 'hidden',
        '#field_name' => $field_name,
        '#field_type' => $this->fieldDefinition->getType(),
        '#primary' => $primary,
        '#secondary' => $secondary,
        '#transition' => 'opacity ' . ($this->getSetting('transition_speed') ?: $defaults['transition_speed']) . ' ' . ($this->getSetting('transition_type') ?: $defaults['transition_type']) . ' 0s',
      ];
    }

    $field_level_access_cacheability = new CacheableMetadata();

    // Try to map the cacheability of the access result that was set at
    // _accessCacheability in getEntitiesToView() to the corresponding render
    // subtree. If no such subtree is found, then merge it with the field-level
    // access cacheability.
    foreach ($items as $delta => $item) {
      // Ignore items for which access cacheability could not be determined in
      // prepareView().
      if (!empty($item->_accessCacheability)) {
        if (isset($elements[$delta])) {
          CacheableMetadata::createFromRenderArray($elements[$delta])
            ->merge($item->_accessCacheability)
            ->applyTo($elements[$delta]);
        }
        else {
          $field_level_access_cacheability = $field_level_access_cacheability->merge($item->_accessCacheability);
        }
      }
    }

    // Apply the cacheability metadata for the inaccessible entities and the
    // entities for which the corresponding render subtree could not be found.
    // This causes the field to be rendered (and cached) according to the cache
    // contexts by which the access results vary, to ensure only users with
    // access to this field can view it. It also tags this field with the cache
    // tags on which the access results depend, to ensure users that cannot view
    // this field at the moment will gain access once any of those cache tags
    // are invalidated.
    $field_level_access_cacheability->merge(CacheableMetadata::createFromRenderArray($elements))
      ->applyTo($elements);

    return $elements;
  }

}
